import { useState, useEffect, useCallback } from "react";
import { useQuery, useMutation } from "@tanstack/react-query";
import { Card } from "@/components/ui/card";
import { MessageList } from "@/components/chat/message-list";
import { MessageInput } from "@/components/chat/message-input";
import { ModelSelector } from "@/components/chat/model-selector";
import { type ModelType, type Message, type Chat, type FileInfo } from "@shared/schema";
import { apiRequest, queryClient } from "@/lib/queryClient";
import { useToast } from "@/hooks/use-toast";
import { Button } from "@/components/ui/button";
import { ImageIcon, Trash2, FileText, FileImage } from "lucide-react";
import { SiOpenai } from "react-icons/si";
import { MistralLogo } from "@/components/icons/MistralLogo";
import { AnthropicLogo } from "@/components/icons/AnthropicLogo";
import { QwenLogo } from "@/components/icons/QwenLogo";
import { DeepseekLogo } from "@/components/icons/DeepseekLogo";
import { Header } from "@/components/ui/header";
import { Footer } from "@/components/ui/footer";
import { WalletButton } from "@/components/chat/wallet-button";
import { SettingsPanel } from "@/components/chat/settings-panel";
import { OxaiLogo } from "@/components/ui/oxai-logo";
import {
  AlertDialog,
  AlertDialogAction,
  AlertDialogCancel,
  AlertDialogContent,
  AlertDialogDescription,
  AlertDialogFooter,
  AlertDialogHeader,
  AlertDialogTitle,
  AlertDialogTrigger,
} from "@/components/ui/alert-dialog";
import { useLocation, useSearch, Link } from "wouter";

const modelIcons: Partial<Record<ModelType, React.ReactNode>> = {
  "gpt-4o-2024-11-20": <SiOpenai className="h-4 w-4" />,
  "claude-3-7-sonnet": <AnthropicLogo className="h-4 w-4" />,
  "claude-3-5-sonnet-20241022": <AnthropicLogo className="h-4 w-4" />,
  "qwen-2.5-max": <QwenLogo className="h-4 w-4" />,
  "deepseek-r1": <DeepseekLogo className="h-4 w-4" />,
  "mistral-small-3.1-24b": <MistralLogo className="h-4 w-4" />,
  "image-generator": <ImageIcon className="h-4 w-4" />,
  "background-remover": <FileImage className="h-4 w-4" />,
  "contract-audit": <FileText className="h-4 w-4" />,
  "deployed-contract-audit": <FileText className="h-4 w-4" />
};

export default function Chat() {
  const { toast } = useToast();
  const [location, setLocation] = useLocation();
  const [search] = useSearch();
  const [chatId, setChatId] = useState<number | null>(null);
  const [currentModel, setCurrentModel] = useState<ModelType | null>(null);
  const [error, setError] = useState<string | null>(null);
  const [hasStartedTyping, setHasStartedTyping] = useState(false);
  const [isClosingChat, setIsClosingChat] = useState(false);

  const { data: messages = [] } = useQuery<Message[]>({
    queryKey: [`/api/chats/${chatId}/messages`],
    enabled: chatId !== null
  });

  const createChat = useMutation({
    mutationFn: async (model: ModelType) => {
      const res = await apiRequest("POST", "/api/chats", { model });
      const chat = await res.json();
      return chat;
    },
    onSuccess: (chat) => {
      setChatId(chat.id);
      setCurrentModel(chat.model);
      setHasStartedTyping(false);
    },
    onError: (error: Error) => {
      console.error("Create chat error:", error);
      toast({
        variant: "destructive",
        title: "Error",
        description: error.message || "Failed to create chat"
      });
    }
  });

  const handleModelSelect = useCallback(async (model: ModelType) => {
    try {
      await createChat.mutateAsync(model);
      // Don't change the URL location when selecting a model
      // This keeps the path (like /private-chat) intact, which maintains the suggestion
    } catch (error) {
      console.error("Error selecting model:", error);
    }
  }, [createChat]);

  // Store the suggested model based on URL, but don't auto-select it
  const [suggestedModel, setSuggestedModel] = useState<ModelType | null>(null);
  
  // Function to get the suggested model based on path
  const getSuggestedModelForPath = (path: string): ModelType | null => {
    // Get path without query params - handle string safely
    const questionMarkIndex = path.indexOf('?');
    const cleanPath = questionMarkIndex !== -1 ? path.substring(0, questionMarkIndex) : path;
    
    if (cleanPath === '/private-chat') {
      return "gpt-4o-2024-11-20";
    } else if (cleanPath === '/code-generator') {
      return "claude-3-7-sonnet";
    }
    
    // Fall back to query param if path doesn't specify a model
    const params = new URLSearchParams(search);
    return params.get('model') as ModelType | null;
  };
  
  // Effect to determine the suggested model based on URL path
  useEffect(() => {
    setSuggestedModel(getSuggestedModelForPath(location));
  }, [location, search]);

  // Only auto-select the model if it comes from a query parameter (old behavior)
  // This maintains backward compatibility with existing links
  useEffect(() => {
    const params = new URLSearchParams(search);
    const modelFromUrl = params.get('model') as ModelType | null;
    
    if (modelFromUrl && !chatId && !currentModel) {
      handleModelSelect(modelFromUrl);
    }
  }, [search, chatId, currentModel, handleModelSelect]);

  const handleNewChat = () => {
    setChatId(null);
    setCurrentModel(null);
    setHasStartedTyping(false);
    setError(null);
    if (chatId) {
      queryClient.removeQueries({ queryKey: [`/api/chats/${chatId}/messages`] });
    }
    queryClient.invalidateQueries({ queryKey: ["/api/chats"] });
    
    // Make sure the suggested model is maintained based on URL path
    setSuggestedModel(getSuggestedModelForPath(location));
  };

  const handleSaveChat = () => {
    setIsClosingChat(false);
    if (messages && messages.length > 0) {
      toast({
        title: "Chat saved",
        description: "The conversation has been saved to your history.",
      });
    }
    handleNewChat();
  };

  const handleDeleteChat = async () => {
    if (!chatId) return;
    try {
      await apiRequest("DELETE", `/api/chats/${chatId}`);
      // Instead of handleNewChat, redirect to home
      setLocation("/");
      toast({
        title: "Chat deleted",
        description: "The conversation has been permanently deleted.",
      });
    } catch (error) {
      toast({
        variant: "destructive",
        title: "Error",
        description: "Failed to delete the conversation.",
      });
    }
  };

  const sendMessage = useMutation({
    mutationFn: async ({ 
      content, 
      enableSearch, 
      uploadedFiles 
    }: { 
      content: string; 
      enableSearch: boolean;
      uploadedFiles?: FileInfo[];
    }) => {
      setError(null);
      if (!chatId) throw new Error("No active chat");

      // Create optimistic message
      const optimisticUserMessage: Message = {
        id: Date.now(),
        chatId,
        role: "user",
        content,
        createdAt: new Date(),
        metadata: uploadedFiles && uploadedFiles.length > 0 ? { uploadedFiles } : null
      };

      queryClient.setQueryData<Message[]>(
        [`/api/chats/${chatId}/messages`],
        (old = []) => [...old, optimisticUserMessage]
      );

      try {
        const res = await apiRequest("POST", `/api/chats/${chatId}/messages`, {
          content,
          enableSearch,
          uploadedFiles // Pass uploaded files metadata if available
        });

        if (!res.ok) {
          throw new Error(await res.text());
        }

        const { userMessage, assistantMessage } = await res.json();

        if (!assistantMessage || !assistantMessage.content) {
          throw new Error("Invalid response format from server");
        }

        if (currentModel) {
          queryClient.invalidateQueries({ queryKey: [`/api/usage/${currentModel}`] });
        }

        queryClient.setQueryData<Message[]>(
          [`/api/chats/${chatId}/messages`],
          (old = []) => {
            const filteredMessages = old.filter(msg => msg.id !== optimisticUserMessage.id);
            return [...filteredMessages, userMessage, assistantMessage];
          }
        );

        return { userMessage, assistantMessage };
      } catch (error) {
        queryClient.setQueryData<Message[]>(
          [`/api/chats/${chatId}/messages`],
          (old = []) => old.filter(msg => msg.id !== optimisticUserMessage.id)
        );

        if (error instanceof Error && error.message.includes("Chat not found")) {
          handleNewChat();
        }
        throw error;
      }
    },
    onError: (error: unknown) => {
      console.error("Send message error:", error);
      let errorMessage = "Failed to send message";

      if (error instanceof Error) {
        try {
          const errorData = JSON.parse(error.message.split(': ')[1]);
          errorMessage = errorData.details || errorData.error || errorMessage;

          if (errorData.requiresApiKey) {
            errorMessage = `${errorMessage}\n\nPlease ensure your OpenRouter API key is properly configured and starts with 'sk-or-v1-'`;
          }
        } catch (e) {
          errorMessage = error.message;
        }
      }

      setError(errorMessage);
      toast({
        variant: "destructive",
        title: "Error",
        description: errorMessage,
        duration: 5000
      });
    }
  });

  const handleSelectChat = (selectedId: number) => {
    queryClient.fetchQuery({
      queryKey: ["/api/chats"]
    }).then((chats: unknown) => {
      const typedChats = chats as Chat[];
      const chat = typedChats.find(chat => chat.id === selectedId);
      if (chat) {
        setChatId(selectedId);
        setCurrentModel(chat.model);
        setHasStartedTyping(true);
        setError(null);
        queryClient.invalidateQueries({ queryKey: [`/api/chats/${selectedId}/messages`] });
      } else {
        handleNewChat();
      }
    });
  };

  const handleImageGeneration = async (prompt: string) => {
    try {
      if (!chatId) return;

      // Add optimistic message for the prompt
      const optimisticMessage = {
        id: Date.now(),
        chatId,
        role: "user" as const,
        content: prompt,
        createdAt: new Date(),
        metadata: null
      };

      queryClient.setQueryData<Message[]>(
        [`/api/chats/${chatId}/messages`],
        (old = []) => [...old, optimisticMessage]
      );

      const response = await apiRequest('POST', '/api/generate-image', { prompt });

      if (!response.ok) {
        throw new Error('Failed to generate image');
      }

      const data = await response.json();

      if (!data.imageUrl) {
        throw new Error('No image URL in response');
      }

      // Add both prompt and response
      queryClient.setQueryData<Message[]>(
        [`/api/chats/${chatId}/messages`],
        (old = []) => {
          // No need to filter for optimistic message in image generation
          return [
            ...old,
            {
              id: Date.now(),
              chatId,
              role: "user" as const,
              content: prompt,
              createdAt: new Date(),
              metadata: null
            },
            {
              id: Date.now() + 1,
              chatId,
              role: "assistant" as const,
              content: `![Generated Image](${data.imageUrl})`,
              createdAt: new Date(),
              metadata: { imageUrl: data.imageUrl }
            }
          ];
        }
      );
    } catch (error) {
      console.error('Error generating image:', error);
      toast({
        variant: "destructive",
        title: "Error",
        description: "Failed to generate image. Please try again.",
      });

      // No optimistic message to remove for image generation
    }
  };

  const handleSendMessage = async (content: string, enableSearch: boolean, uploadedFiles?: FileInfo[]) => {
    if (currentModel === 'image-generator') {
      await handleImageGeneration(content);
    } else {
      try {
        await sendMessage.mutateAsync({ content, enableSearch, uploadedFiles });
      } catch (error) {
        if (error instanceof Error && !error.message.includes("Chat not found")) {
          toast({
            title: "Error",
            description: "Failed to send message. Please try again.",
            variant: "destructive"
          });
        }
      }
    }
  };

  return (
    <div className="min-h-screen bg-background flex flex-col">
      <div className="flex-1 flex max-w-[1800px] mx-auto w-full">
        {!currentModel ? (
          <div className="flex-1 flex flex-col">
            {/* Unified Header for model selector page */}
            <Header mode="landing" />
            <div className="flex-grow flex items-center justify-center">
              <ModelSelector onSelect={handleModelSelect} suggestedModel={suggestedModel} />
            </div>
            <Footer />
          </div>
        ) : (
          <div className="flex-1 flex flex-col h-screen overflow-hidden">
            {/* Unified Header */}
            <Header 
              mode="chat" 
              model={currentModel}
              chatId={chatId}
              onSelectChat={handleSelectChat}
              onNewChat={handleNewChat}
              onDeleteChat={handleDeleteChat}
              onSaveChat={handleSaveChat}
            />
            
            {/* Main content with padding for fixed header */}
            <main className="flex-1 flex flex-col min-h-0 overflow-hidden pt-16 px-2 sm:px-4">
              <div className="flex-1 overflow-y-auto pb-16 md:pb-0">
                <MessageList
                  messages={messages}
                  loading={sendMessage.isPending}
                  error={error || undefined}
                  onRetry={error ? () => sendMessage.reset() : undefined}
                  hasStartedTyping={hasStartedTyping}
                  model={currentModel}
                  chatId={chatId || 0}
                />
              </div>
              <div className="fixed bottom-0 left-0 right-0 md:relative">
                <MessageInput
                  onSend={handleSendMessage}
                  disabled={sendMessage.isPending}
                  onStartTyping={() => setHasStartedTyping(true)}
                />
              </div>
            </main>
          </div>
        )}
      </div>
    </div>
  );
}